"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs = tslib_1.__importStar(require("fs-extra"));
const path = tslib_1.__importStar(require("path"));
const utils_1 = require("./utils");
function throwIfTemplatePathDoesntExist(templatePath) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (!(yield fs.pathExists(templatePath)))
            throw Error(`template ${templatePath} does not exist.`);
    });
}
function getDestinationFilePath({ templatePath, destinationPath, file, data }) {
    const renderedPath = utils_1.getRenderedPath(file, templatePath, destinationPath);
    return utils_1.getTemplatedFilePath(renderedPath, data);
}
exports.getDestinationFilePath = getDestinationFilePath;
function createTemplate({ templatePath, templateFile, destinationPath, data }) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield throwIfTemplatePathDoesntExist(templatePath);
        const template = yield utils_1.renderTemplateFile(templatePath, templateFile, data);
        let newPath = getDestinationFilePath({
            templatePath,
            destinationPath,
            file: template.fullPath,
            data
        });
        yield fs.writeFile(newPath, template.content);
    });
}
exports.createTemplate = createTemplate;
function renderTemplates({ templatePath, templatePattern, destinationPath, data }) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield throwIfTemplatePathDoesntExist(templatePath);
        const templateFiles = yield utils_1.search(path.join(templatePath, templatePattern));
        for (const templateFile of templateFiles) {
            yield createTemplate({ templatePath, templateFile, destinationPath, data });
        }
    });
}
exports.renderTemplates = renderTemplates;
function renderRegularFiles({ templatePath, regularPattern, destinationPath, data, ignorePattern }) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield throwIfTemplatePathDoesntExist(templatePath);
        const files = yield utils_1.search(path.join(templatePath, regularPattern), {
            nodir: true,
            ignore: ignorePattern
        });
        for (const file of files) {
            let newPath = getDestinationFilePath({
                templatePath,
                destinationPath,
                file,
                data
            });
            yield fs.copy(file, newPath);
        }
    });
}
exports.renderRegularFiles = renderRegularFiles;
function renderFolders({ templatePath, folderPattern, destinationPath, data, ignorePattern }) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield throwIfTemplatePathDoesntExist(templatePath);
        const folders = yield utils_1.search(path.join(templatePath, folderPattern), {
            ignore: ignorePattern
        });
        // const folders = await search(templateFoldersPattern, {
        //   ignore: patterns.projectTemplatePartialFiles
        // })
        for (const folder of folders) {
            let newPath = getDestinationFilePath({
                templatePath,
                destinationPath,
                file: folder,
                data
            });
            yield fs.ensureDir(newPath);
        }
    });
}
exports.renderFolders = renderFolders;
function renderMacros({ templatePath, macroPattern, destinationPath, data }) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield throwIfTemplatePathDoesntExist(templatePath);
        const macroPaths = yield utils_1.search(path.join(templatePath, macroPattern));
        for (const macroPath of macroPaths) {
            //TODO handle if macro is missing or malformed
            const macro = require(path.resolve(macroPath)).default;
            yield macro({
                data,
                search: utils_1.search,
                destinationPath,
                renderTemplateFile: utils_1.renderTemplateFile,
                renderTemplateString: utils_1.renderTemplateString,
                getTemplatedFilePath: utils_1.getTemplatedFilePath
            });
        }
    });
}
exports.renderMacros = renderMacros;
function renderPartials({ templatePath, partialPattern, partialsPath, destinationPath, data }) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield throwIfTemplatePathDoesntExist(templatePath);
        const partialPaths = yield utils_1.search(path.join(templatePath, partialPattern));
        for (const partialPath of partialPaths) {
            let newPath = getDestinationFilePath({
                templatePath,
                destinationPath,
                file: partialPath,
                data
            }).replace(".partial", "");
            const partialName = path.basename(newPath);
            const partialTemplatePath = path.join(partialsPath, partialName);
            yield fs.copy(partialTemplatePath, newPath);
        }
    });
}
exports.renderPartials = renderPartials;
function renderTemplate({ templatePath, destinationPath, data = {}, partialsPath = "partials", templatePattern = "**/*.template", regularPattern = "**/!(*.template|*.macro.js|*.partial)", macroPattern = "**/*.macro.js", partialPattern = "**/*.partial", folderPattern = "**/", ignorePattern }) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield throwIfTemplatePathDoesntExist(templatePath);
        const getMeasuredTime = utils_1.measureTime();
        yield renderFolders({
            templatePath,
            folderPattern,
            destinationPath,
            data,
            ignorePattern
        });
        yield Promise.all([
            yield renderTemplates({
                templatePath,
                templatePattern,
                destinationPath,
                data
            }),
            yield renderRegularFiles({
                templatePath,
                regularPattern,
                destinationPath,
                data,
                ignorePattern
            }),
            yield renderPartials({
                templatePath,
                partialPattern,
                partialsPath,
                destinationPath,
                data
            })
        ]);
        yield renderMacros({ templatePath, macroPattern, destinationPath, data });
        const createdIn = getMeasuredTime();
        // console.log(`"${templatePath}" -> "${destinationPath}" ${createdIn}ms`)
        return {
            templatePath,
            destinationPath,
            createdIn
        };
    });
}
exports.renderTemplate = renderTemplate;
//# sourceMappingURL=index.js.map